unit IDGHierarchy;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DSGNINTF, about;

type
  TIDGHierarchy = class;
  THierarchyItem = class;

  TEventItemClick = procedure ( Sender : TObject; Item : THierarchyItem ) of object;

  TItemsProperty = class(TClassProperty)
  public
    function  GetAttributes : TPropertyAttributes ; override;
    procedure Edit; override;
    procedure SetValue(const Value: string); override;
  end;


  THierarchyItem = class
  private
    FParentItem : THierarchyItem;

    FRect : TRect;
    FText : string;
    FShow : boolean;
    FExpanded : boolean;
    FCurrent : boolean;

    FIndex : Integer;

  public
    constructor Create( Parent : THierarchyItem);
    destructor Destroy; override;

    property Text : string read FText write FText;
    property Parent : THierarchyItem read FParentItem write FParentItem;
    property Show : boolean read FShow write FShow;
    property Expanded : boolean read FExpanded write FExpanded;
    property Rect : TRect read FRect write FRect;
    property Current : boolean read FCurrent write FCurrent;
    property IndexItem : Integer read FIndex write FIndex;
  end;


  THierarchyItems = class(TPersistent)
  private
    FListItem : TList;
    FIDGHierarchy : TIDGHierarchy;

    procedure DefineProperties(Filer: TFiler);override;
    procedure ReadBinaryData(Stream: TStream);
    procedure WriteBinaryData(Stream: TStream);

  protected

  public
    constructor Create( AOwner : TIDGHierarchy);virtual;
    destructor Destroy; override;

    procedure Expanded( Item : THierarchyItem );
    procedure Collapse( Item : THierarchyItem );
    function IsInside( X, Y : Integer) : THierarchyItem;

    function FindCurrent : THierarchyItem;
    function FindByParent( Parent : THierarchyItem) : TList;
    procedure Clear;

    function  AddItem( Parent : THierarchyItem; Text : string) : THierarchyItem;
    procedure DeleteItem( item : THierarchyItem );

    procedure PaintItems( ParentItems : THierarchyItem; var TopLine : Integer;
                          LeftLine  : Integer);

    function GetInvalidateRect( ParentItems : THierarchyItem; TopLine : Integer ) : TRect;
    function GetParent( Item : THierarchyItem ) : THierarchyItem;

  end;

  { TIDGHierarchy }
  TIDGHierarchy = class( TGraphicControl )
  private
    FItems : THierarchyItems;

    FLevelMargin : Integer;
    FLeftMargin : Integer;
    FTopMargin : Integer;

    FPicture : TPicture;
//    FTransparent : boolean;

    FFontColor   : TColor;
    FCurrentFontColor : TColor;

    FItemClick : TEventItemClick;

    FFontChild : TFont;
    FFont      : TFont;

    FUpdateBackground : boolean;

//    function GetFont : TFont;
    procedure SetFont( Value : TFont);
    procedure SetFontChild( Value : TFont);
    procedure SetPicture(Value: TPicture);

    procedure SetTopMargin(Value : Integer);
    procedure SetLeftMargin(Value : Integer);
    procedure SetLevelMargin(Value : Integer);
    procedure SetFontColor( Value : TColor);
    procedure SetCurrentFontColor(Value : TColor);
//    function  GetTransparent : boolean;
//    procedure SetTransparent(Value : boolean);

  protected
    procedure Paint;override;
    procedure WMMove(var Message : TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonDown(var Message : TWMLButtonDown); message WM_LBUTTONDOWN;

  public

    constructor Create( AOwner : TComponent); override;
    destructor  Destroy; override;
//    property TransparentPanel : boolean read GetTransparent write SetTransparent;

  published

    property LevelMargin : Integer read FLevelMargin write SetLevelMargin;
    property LeftMargin : Integer read FLeftMargin write SetLeftMargin;
    property TopMargin : Integer read FTopMArgin write SetTopMargin;

    property Align;
    property Font : TFont read FFont write SetFont;
    property FontChild : TFont read FFontChild write SetFontChild;

    property CurrentFontColor : TColor read FCurrentFontColor write SetCurrentFontColor;
    property FontColor : TColor read FFontColor write SetFontColor;

    property Picture : TPicture read FPicture write SetPicture;

    property OnItemClick : TEventItemClick read FItemClick write FItemClick;
    property Visible;
    property Items : THierarchyItems read FItems write FItems;

    property UpdateBackground : boolean read FUpdateBackground write FUpdateBackground;

  end;

procedure Register;

implementation

uses ItemsHierarchy;

//-------------------
function TItemsProperty.GetAttributes : TPropertyAttributes ;
begin
   Result := [paMultiSelect, paDialog, paAutoUpdate];
end;

procedure TItemsProperty.Edit;
var
  dialog : TfmItemsHierarchy;
  hiList : TIDGHierarchy;
begin

  hiList := TIDGHierarchy( GetComponent(0));

  dialog := TfmItemsHierarchy.CreateDiag(Application, hiList.Items);

  try
    if Dialog.ShowModal = IDOK then begin
      hiList.Invalidate;
      SetValue('New');
    end;
  finally
    Dialog.Free;
  end;
end;

procedure TItemsProperty.SetValue(const Value: string);
begin
  SetStrValue(Value);
end;



//--  THierarchyItem -------------------------------

constructor THierarchyItem.Create( Parent : THierarchyItem);
begin
  inherited Create;
  FParentItem := Parent;
  FExpanded := false;
  FCurrent := false;
  if ( Parent = nil ) then FShow := true;

end;

destructor THierarchyItem.Destroy;
begin
  inherited;
end;

// ---------- THierarchyItems --------------------------

constructor THierarchyItems.Create( AOwner : TIDGHierarchy);
begin
  inherited Create;
  FIDGHierarchy := AOwner;
  FListItem := TList.Create();
end;

destructor THierarchyItems.Destroy;
begin
  Clear;
  FListItem.Free;
  inherited;
end;

procedure THierarchyItems.PaintItems( ParentItems : THierarchyItem; var TopLine : Integer;
                                      LeftLine  : Integer);
var
  list : TList;
  i : Integer;
  item : THierarchyItem;
  width, height : integer;
  left : integer;
  rect : TRect;
  OldMode : Integer;
  OldFont : TFont;
begin
  left := LeftLine;

  list := FindByParent( ParentItems );
  OldMode := SetBkMode( FIDGHierarchy.Canvas.Handle, TRANSPARENT );

  for i := 0 to list.Count - 1 do begin
    item := THierarchyItem (list[i]);

    OldFont := FIDGHierarchy.Canvas.Font;
    if ( Assigned( item.Parent )) then
      FIDGHierarchy.Canvas.Font := FIDGHierarchy.FFontChild
    else
      FIDGHierarchy.Canvas.Font := FIDGHierarchy.FFont;

    if (item.FCurrent) then
      FIDGHierarchy.Canvas.Font.Color := FIDGHierarchy.CurrentFontColor
    else
      FIDGHierarchy.Canvas.Font.Color := FIDGHierarchy.FontColor;

    width := FIDGHierarchy.Canvas.TextWidth( item.Text );
    height := FIDGHierarchy.Canvas.TextHeight( item.Text );

    Rect.Left  := LeftLine;
    Rect.Top   := TopLine;
    Rect.Right := LeftLine + width;
    Rect.Bottom := topLine + height;

    item.Rect := rect;


    DrawText( FIDGHierarchy.Canvas.Handle, PChar(item.Text),
              Length(item.Text), Rect, DT_LEFT + DT_VCENTER );

    TopLine := TopLine + height;

    FIDGHierarchy.Canvas.Font := OldFont;

    if (item.Expanded) then
      PaintItems( item, TopLine, Left + FIDGHierarchy.FLevelMargin);

  end;

  SetBkMode( FIDGHierarchy.Canvas.Handle, OldMode );
  list.Free;
end;

function THierarchyItems.GetInvalidateRect( ParentItems : THierarchyItem; TopLine : Integer ) : TRect;
var
  list, ChildList : TList;
  i, IndexBegin : Integer;
  item, ChildItem : THierarchyItem;
  iHeight : integer;
  left : integer;
  rect,rect1 : TRect;
  OldFont : TFont;
  iTop : Integer;
begin
  left := ParentItems.Rect.Left;
  iTop := TopLine;

  Rect.Left := left;
  Rect.Right := FIDGHierarchy.Width;;
  Rect.Top := iTop;

  list := FindByParent( ParentItems.FParentItem);
  IndexBegin := list.IndexOf(ParentItems);

  for i := IndexBegin to list.Count - 1 do begin
    item := THierarchyItem (list[i]);

    OldFont := FIDGHierarchy.Canvas.Font;
    if ( Assigned( item.Parent )) then
      FIDGHierarchy.Canvas.Font := FIDGHierarchy.FFontChild
    else
      FIDGHierarchy.Canvas.Font := FIDGHierarchy.FFont;

    iHeight := FIDGHierarchy.Canvas.TextHeight( item.Text );

    iTop := iTop + iHeight;
    FIDGHierarchy.Canvas.Font := OldFont;

    if (item.Expanded) then begin
      Childlist := FindByParent( Item );
      if Childlist.Count > 0 then begin
        ChildItem := ChildList[0];
        Rect1 := GetInvalidateRect( ChildItem, iTop);
        iTop := Rect1.Bottom;
        Childlist.Free;
      end;
    end;
  end;

  Rect.Bottom := iTop + iHeight;
  Result := rect;

  list.Free;
end;

function THierarchyItems.GetParent( Item : THierarchyItem ) : THierarchyItem;
begin
  if Assigned(Item.FParentItem) then
    Result := GetParent( Item.FParentItem )
  else
    Result := Item;
end;

function THierarchyItems.FindByParent( Parent : THierarchyItem) : TList;
var
  I : Integer;
  item : THierarchyItem;
begin
  Result := TList.Create;
  for i := 0 to FListItem.Count - 1 do begin
    item := THierarchyItem( FListItem[i] );
    if (item.Parent = Parent) then
      Result.Add(item);
  end;
end;

procedure THierarchyItems.Clear;
var
  item : THierarchyItem;
begin
  while FListItem.Count > 0 do begin
    item := THierarchyItem( FListItem[0] );
    item.Free;
    FListItem.Delete(0);
  end;
end;

function THierarchyItems.AddItem( Parent : THierarchyItem; Text : string) : THierarchyItem;
var
  item : THierarchyItem;
begin
  item := THierarchyItem.Create(Parent);
  item.Text := Text;
  FListItem.Add(item);
  Result := item;
end;

procedure THierarchyItems.DeleteItem( item : THierarchyItem );
var
  list : TList;
  i : Integer;
  itemChild : THierarchyItem;
begin

  list := FindByParent( item );
  for i := 0 to list.Count - 1 do begin
    itemChild := THierarchyItem (list[i]);
    DeleteItem(itemChild);
  end;
  list.Clear;
  list.Free;

  FListItem.Delete( FListItem.IndexOf(item));
  item.Free;

end;

function THierarchyItems.IsInside( X, Y : Integer) : THierarchyItem;
var
  i : Integer;
  item : THierarchyItem;
begin
  Result := nil;
  for i := 0 to FListItem.Count - 1 do begin
    item := THierarchyItem(FListItem[i]);
    if (item.Show) and ( item.Rect.Left < X ) and ( item.Rect.Right > X )
      and ( item.Rect.Top < Y) and ( item.Rect.Bottom > Y ) then begin
        Result := item;
    end;
  end;
end;

function THierarchyItems.FindCurrent : THierarchyItem;
var
  i : Integer;
  item : THierarchyItem;
begin
  Result := nil;
  for i := 0 to FListItem.Count - 1 do begin
    item := THierarchyItem(FListItem[i]);
    if (item.Current) then begin
        Result := item;
    end;
  end;
end;

procedure THierarchyItems.Expanded( Item : THierarchyItem );
var
  list : TList;
  ItemChild : THierarchyItem;
  i : Integer;
  rect : Trect;
begin
  if not ( Item.Expanded ) then begin
    list := FindByParent( Item );
    for i := 0 to list.Count - 1 do begin
      ItemChild := THierarchyItem( list[i] );
      ItemChild.FShow := true;
    end;
    if (list.Count > 0) then begin
      Item.Expanded := true;
      rect := GetInvalidateRect( GetParent(Item), Item.Rect.Top );

      if FIDGHierarchy.FUpdateBackground then
        InvalidateRect( FIDGHierarchy.Parent.Handle, @rect, true)
      else
        InvalidateRect( FIDGHierarchy.Parent.Handle, @rect, false);

    end;
    list.Free;
  end;
end;

procedure THierarchyItems.Collapse( Item : THierarchyItem );
var
  list : TList;
  ItemChild : THierarchyItem;
  i : Integer;
  rect : TRect;
begin
  if ( Item.Expanded ) then begin
    list := FindByParent( Item );
    for i := 0 to list.Count - 1 do begin
      ItemChild := THierarchyItem( list[i] );
      ItemChild.FShow := false;
    end;
    if (list.Count > 0) then begin
      rect := GetInvalidateRect( GetParent(Item), Item.Rect.Top );
      Item.Expanded := false;

      if FIDGHierarchy.FUpdateBackground then
        InvalidateRect( FIDGHierarchy.Parent.Handle, @rect, true )
      else
        InvalidateRect( FIDGHierarchy.Parent.Handle, @rect, false );

    end;
    list.Free;
  end;
end;

procedure THierarchyItems.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Text', ReadBinaryData, WriteBinaryData,
    true);
end;

procedure THierarchyItems.ReadBinaryData(Stream: TStream);
var
  I : integer;
  item : THierarchyItem;
  Index : Integer;
  temp : Integer;
  Str : string;
  ParentInt : Integer;
  Count : Integer;
begin
  Clear;
  with Stream do
  begin
    ReadBuffer( Count, SizeOf(Count));
    for I := 0 to Count - 1 do begin
      item := THierarchyItem.Create(nil);
      with item do begin
        ReadBuffer(Index, SizeOf(Index));
        IndexItem := Index;
        ReadBuffer(Temp, SizeOf(Temp));
        SetLength(Str, Temp);
        ReadBuffer( PChar(Str)^, Temp);
        Text := str;
        ReadBuffer(ParentInt, SizeOf(ParentInt));
        if ( ParentInt > -1 ) and (ParentInt < FListItem.Count ) then
          Parent := THierarchyItem( FListItem[ParentInt]);
      end;
      FListItem.Add(item);
    end;
  end;
end;

procedure THierarchyItems.WriteBinaryData(Stream: TStream);
var
  I: Integer;
  Temp: Integer;
  item : THierarchyItem;
  ParentInt : Integer;
begin
  with Stream do begin
    Temp := FListItem.Count;
    WriteBuffer(Temp, SizeOf(Temp));
    for I := 0 to Temp - 1  do begin
      item := FListItem[i];
      with item do begin
        Temp := IndexItem;
        WriteBuffer(Temp, SizeOf(Temp));
        Temp := Length(Text);
        WriteBuffer(Temp, SizeOf(Temp));
        WriteBuffer(PChar(Text)^, Length(Text));
        if ( item.Parent = nil) then
          ParentInt := -1
        else
          ParentInt := FListItem.IndexOf(item.Parent);
        WriteBuffer(ParentInt, SizeOf(ParentInt));
      end;
    end;
  end;
end;


// ------------ TIDGHierarchy -----------------------

constructor TIDGHierarchy.Create( AOwner : TComponent);
begin
//  TransparentPanel := true;
  inherited Create(AOwner);
  FItems := THierarchyItems.Create( Self );
  Top := 0;
  Left := 0;
  Width := 20;
  Height := 20;
  LevelMargin := 15;
  LeftMargin := 20;

  FPicture := TPicture.Create;
  FFont := TFont.Create;
  FFontChild := TFont.Create;

end;

destructor TIDGHierarchy.Destroy;
begin
  FItems.Free;
  FPicture.Free;
  FFont.Free;
  FFontChild.Free;
  inherited;
end;

procedure TIDGHierarchy.Paint;
var

  TopLine : Integer;
  TopBrushLine, LeftBrushLine : Integer;
  countH, countV : Integer;
  i, j : Integer;
begin
//  inherited;

  if (FPicture.Width > 0 ) then begin
    countH := Round( Width / FPicture.Width ) + 1;
    countV := Round( Height / FPicture.Height ) + 1;

    TopBrushLine := 0;
    for i := 1 to countV do begin
      leftBrushLine := 0;
      for j := 1 to countH do begin
        Canvas.Draw(leftBrushLine, topBrushLine, FPicture.Graphic);
        leftBrushLine := leftBrushLine + FPicture.Width;
      end;
      topBrushLine := topBrushLine + FPicture.Height;
    end;
  end;

  TopLine := TopMargin;
  FItems.PaintItems( nil, TopLine, LeftMargin );
end;

{function TIDGHierarchy.GetFont : TFont;
begin
  Result := Canvas.Font;
end;}

procedure TIDGHierarchy.SetFont( Value : TFont);
begin
  FFont.Assign( Value );
  Invalidate;
end;

procedure TIDGHierarchy.SetFontChild( Value : TFont);
begin
  FFontChild.Assign( Value );
  Invalidate;
end;

procedure TIDGHierarchy.SetTopMargin(Value : Integer);
begin
  FTopMargin := Value;
  Invalidate;
end;

procedure TIDGHierarchy.SetLeftMargin(Value : Integer);
begin
  FLeftMargin := Value;
  Invalidate;
end;

procedure TIDGHierarchy.SetLevelMargin(Value : Integer);
begin
  FLevelMargin := Value;
  Invalidate;
end;

procedure TIDGHierarchy.SetFontColor( Value : TColor);
begin
  FFontColor := Value;
  Invalidate;
end;

procedure TIDGHierarchy.SetCurrentFontColor(Value : TColor);
begin
  FCurrentFontColor := Value;
  Invalidate;
end;

{function TIDGHierarchy.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TIDGHierarchy.SetTransparent(Value : boolean);
begin
  if TransparentPanel <> Value then begin
    if Value then ControlStyle := ControlStyle - [csOpaque]
    else ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;}

procedure TIDGHierarchy.WMMove( var Message : TWMMouseMove);
var
  item, OldItem : THierarchyItem;
  rect : TRect;
  OldMode : Integer;
begin

  OldMode := SetBkMode( Canvas.Handle, TRANSPARENT );
  OldItem := FItems.FindCurrent;

  item := FItems.IsInside( Message.XPos, Message.YPos );

  if ( item <> nil ) and ( item <> OldItem ) then begin

//    OldFont := Canvas.Font;

    if ( OldItem <> nil) then begin
      if ( Assigned( Olditem.Parent )) then
        Canvas.Font := FFontChild
      else
        Canvas.Font := FFont;

      Canvas.Font.Color := FFontColor;
      rect := OldItem.Rect;
      DrawText( Canvas.Handle, PChar(OldItem.Text),
              Length(OldItem.Text), rect, DT_LEFT + DT_VCENTER );
      OldItem.Current := false;
    end;

    if ( Assigned( item.Parent )) then
      Canvas.Font := FFontChild
    else
      Canvas.Font := FFont;

    Canvas.Font.Color := FCurrentFontColor;
    rect := item.Rect;

    DrawText( Canvas.Handle, PChar(item.Text),
              Length(item.Text), rect, DT_LEFT + DT_VCENTER );

    item.Current := true;
//    Canvas.Font := OldFont;
  end;

  if ( item = nil ) and ( OldItem <> nil) then begin

      if ( Assigned( Olditem.Parent )) then
        Canvas.Font := FFontChild
      else
        Canvas.Font := FFont;

      Canvas.Font.Color := FFontColor;
      rect := OldItem.Rect;
      DrawText( Canvas.Handle, PChar(OldItem.Text),
              Length(OldItem.Text), rect, DT_LEFT + DT_VCENTER );
      OldItem.Current := false;
  end;

  SetBkMode( Canvas.Handle, OldMode );
end;

procedure TIDGHierarchy.WMLButtonDown(var Message : TWMLButtonDown);
var
  item : THierarchyItem;
begin

  item := FItems.IsInside( Message.XPos, Message.YPos );
  if (item<> nil) then begin
    if not (item.Expanded) then
      FItems.Expanded(item)
    else
      FItems.Collapse(item);
    if Assigned(FItemClick) then
      FItemClick( Self, Item );
  end;

end;

procedure TIDGHierarchy.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
  Invalidate;
end;

//- ----------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('IDG', [TIDGHierarchy]);
  RegisterPropertyEditor(TypeInfo(THierarchyItems),TIDGHierarchy,'Items',TItemsProperty);
  RegisterComponentEditor( TIDGHierarchy, TAboutEditor );
end;

end.
