unit ItemsHierarchy;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, IDGHierarchy;

type

  TfmItemsHierarchy = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    tvItems: TTreeView;
    bitNewItem: TButton;
    butNewSubItem: TButton;
    butDelete: TBitBtn;
    butOK: TButton;
    butCancel: TButton;
    edID: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    edName: TEdit;
    procedure bitNewItemClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure butNewSubItemClick(Sender: TObject);
    procedure butDeleteClick(Sender: TObject);
    procedure edNameChange(Sender: TObject);
    procedure tvItemsChange(Sender: TObject; Node: TTreeNode);
    procedure tvItemsDeletion(Sender: TObject; Node: TTreeNode);
    procedure edIDChange(Sender: TObject);
    procedure butOKClick(Sender: TObject);
  private
    { Private declarations }
    FItems : THierarchyItems;
    procedure FillTreeItems( node : TTreeNode; item : THierarchyItem );
    procedure FillHierarchyItems( node : TTreeNode; item : THierarchyItem );
  public
    { Public declarations }
    constructor CreateDiag( AOwner : TComponent; items : THierarchyItems);
  end;

var
  fmItemsHierarchy: TfmItemsHierarchy;

implementation


{$R *.DFM}

constructor TfmItemsHierarchy.CreateDiag( AOwner : TComponent; items : THierarchyItems);
begin
  inherited Create(AOwner);
  FItems := items;
  FillTreeItems(nil, nil);
end;


procedure TfmItemsHierarchy.FillTreeItems( node : TTreeNode; item : THierarchyItem );
var
  list : TList;
  i : Integer;
  ID : PInteger;
  Childitem : THierarchyItem;
  text : string;
  nodeChild : TTreeNode;
begin
  list := FItems.FindByParent( item );
  for i := 0 to list.Count - 1 do begin
    ChildItem := list[i];
    New(ID);
    ID^ := ChildItem.IndexItem;
    text := ChildItem.Text;
    if (node = nil) then
      nodeChild := tvItems.Items.AddObject(nil, text, ID)
    else
      nodeChild := tvItems.Items.AddChildObject( node, text, ID);

    FillTreeItems( nodeChild, ChildItem );
  end;
  list.Free;

end;

procedure TfmItemsHierarchy.FillHierarchyItems( node : TTreeNode; item : THierarchyItem );
var
  i : Integer;
  ID : PInteger;
  Childitem : THierarchyItem;
  nodeChild : TTreeNode;
begin

  for i := 0 to tvItems.Items.Count - 1 do begin
    nodeChild := tvItems.Items.Item[i];
    if (nodeChild.Parent = node) then begin
      ID := nodeChild.Data;

      ChildItem := FItems.AddItem(item, nodeChild.Text);
      ChildItem.IndexItem := ID^;

      FillHierarchyItems( nodeChild, ChildItem );
    end;
  end;
end;

procedure TfmItemsHierarchy.bitNewItemClick(Sender: TObject);
var
  ID : PInteger;
  node : TTreeNode;
begin
  if (tvItems.Selected = nil) or
     (tvItems.Selected.Parent = nil) then begin
    New(ID);
    ID^ := 0;
    node := tvItems.Items.AddObject(nil, '', ID);
  end
  else begin
    New(ID);
    ID^ := 0;
    node := tvItems.Items.AddChildObject( tvItems.Selected.Parent, '', ID);
  end;
  tvItems.Selected := node;

end;

procedure TfmItemsHierarchy.FormShow(Sender: TObject);
begin
{  if ( tvItems.Selected = nil) then begin
    butNewSubItem.Enabled := false;
    butDelete.Enabled := false;
  end
  else begin
    butNewSubItem.Enabled := true;
    butDelete.Enabled := true;
  end;}

end;

procedure TfmItemsHierarchy.butNewSubItemClick(Sender: TObject);
var
  ID : PInteger;
  node : TTreeNode;
begin
  if not (tvItems.Selected = nil) then begin
    New(ID);
    ID^ := 0;
    node := tvItems.Items.AddChildObject( tvItems.Selected, '', ID);
    tvItems.Selected.Expand(true);
    tvItems.Selected := node;
  end;
end;

procedure TfmItemsHierarchy.butDeleteClick(Sender: TObject);
begin
  if not (tvItems.Selected = nil) then begin
    tvItems.Items.Delete( tvItems.Selected);
  end;
end;

procedure TfmItemsHierarchy.edNameChange(Sender: TObject);
begin
  if not (tvItems.Selected = nil) then begin
    tvItems.Selected.Text := edName.Text;
  end;
end;

procedure TfmItemsHierarchy.tvItemsChange(Sender: TObject;
  Node: TTreeNode);
var
  i : PInteger;
begin
  if not (tvItems.Selected = nil) then begin
    edName.TExt := tvItems.Selected.Text;
    i := tvItems.Selected.Data;
    edId.Text := IntToStr(i^);
  end;
end;

procedure TfmItemsHierarchy.tvItemsDeletion(Sender: TObject;
  Node: TTreeNode);
var
  ID : PInteger;
begin
  ID := Node.Data;
  Dispose(ID);
end;

procedure TfmItemsHierarchy.edIDChange(Sender: TObject);
var
  I : PInteger;
  I1 : Integer;
begin
  if not (tvItems.Selected = nil) then begin
    if (edID.Text = '' ) then
      I1 := 0
    else
      I1 := StrToInt(edID.Text);
    I := tvItems.Selected.Data;
    I^ := I1;
  end;
end;

procedure TfmItemsHierarchy.butOKClick(Sender: TObject);
begin
  FItems.Clear;
  FillHierarchyItems( nil, nil );
end;

end.
